package pvc

import (

	//	"github.com/ankitrgadiya/operatorlib/pkg/meta"
	"context"

	"github.com/ankitrgadiya/operatorlib/pkg/interfaces"

	"github.com/pkg/errors"
	corev1 "k8s.io/api/core/v1"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"

	"github.com/presslabs/controller-util/syncer"
)

func (c Conf) Mutate(obj *corev1.PersistentVolumeClaim) error {
	var om *metav1.ObjectMeta
	var accessModes []corev1.PersistentVolumeAccessMode
	var resources corev1.ResourceRequirements
	var err error

	om, err = c.Meta.MutateObjectMeta()

	if err != nil {
		return errors.Wrap(err, "failed to generate objectmeta")
	}
	if c.GenAccessModesFunc != nil {
		accessModes, err = c.GenAccessModesFunc(c.Meta.Instance)
		if err != nil {
			return errors.Wrap(err, "failed to generate pvc accessmodes")
		}
	}
	if c.GenResourcesFunc != nil {
		resources, err = c.GenResourcesFunc(c.Meta.Instance)
		if err != nil {
			return errors.Wrap(err, "failed to generate pvc accessmodes")
		}
	}

	obj.ObjectMeta = *om
	obj.Spec = corev1.PersistentVolumeClaimSpec{
		AccessModes:      accessModes,
		Resources:        resources,
		StorageClassName: c.StorageClassName,
	}
	return nil
}

//func NewPVCSyncer(c Conf) syncer.Interface {
//	client := c.Reconcile.GetClient()
//	scheme := c.Reconcile.GetScheme()

//	obj := &corev1.PersistentVolumeClaim{
//		ObjectMeta: metav1.ObjectMeta{
//			Name:      c.Name,
//			Namespace: c.Namespace,
//		},
//	}

//	return syncer.NewObjectSyncer("App PVC", c.Instance, obj, client, scheme, func() error {
//		if err := c.Mutate(obj); err != nil {
//			return err
//		}
//		return nil
//	})
//}

func SyncObject(c Conf, ctx context.Context) error {

	client := c.Reconcile.GetClient()
	scheme := c.Reconcile.GetScheme()
	recorder := c.Reconcile.GetRecorder()

	obj := &corev1.PersistentVolumeClaim{
		ObjectMeta: metav1.ObjectMeta{
			Name:      c.Meta.Name,
			Namespace: c.Meta.Namespace,
		},
	}

	PVCSyncer := syncer.NewObjectSyncer("App PVC", c.Meta.Instance, obj, client, scheme, func() error {
		if err := c.Mutate(obj); err != nil {
			return err
		}
		return nil
	})

	if err := syncer.Sync(ctx, PVCSyncer, recorder); err != nil {
		return err
	}
	return nil
}

func (c Conf) GetInstance() interfaces.Object {
	return c.Meta.Instance
}
func (c Conf) GetName() string {
	return c.Meta.Name
}
func (c Conf) GetNamespace() string {
	return c.Meta.Namespace
}
